%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% main.m
% Example file for using the time-domain implementation of the
% PemAFC-based feedback cancellation algorithm
% Modified by Linh Tran based on Ann Spriet's code 

% PEMSC-APA and swPEMSC
% Author: Linh Tran
% Date: March 2021 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all;
close all;
addpath('data_files');
% addpath('CommonFunc\TD');
addpath('CommonFunc');
addpath('CommonFunc\Obj_estimate');
% addpath('feedback cancellation_S');

%% Set Variables
prob_sig = 0;         % select among 0) without probe signal, 1)with probe signal as a white noise 
in_sig = 2;           % 0) white noise, 1) speech weighted noise, 2) real speech, 3) music

fs = 16000;          % sampling frequency
N = 60*fs;            % total number of samples

Kdb = 30;             % gain of forward path in dB
K = 10^(Kdb/20);        
d_k = 96;             % delay of the forward path K(q) in samples 
d_fb = 1;            % delay of the feedback cancellation path in samples
Lg_hat = 64;          % the full length of adaptive filter
SNR = 20;               % amplified signal to injected noise ratio

% Fixed Stepsize
mu = 0.0008;         % fixed step size 
mu1 = 0.0008;                 
mu2 = 0.8;
delta = 1e-6;

% APA parameter
P = 6;              % Projection order

%%%%%%%%%%%%%%%%%%%%%%%%%
%Settings Feedback path %
%%%%%%%%%%%%%%%%%%%%%%%%%
%% Feedback path  

load('mFBPathIRs16kHz_FF.mat');
E = mFBPathIRs16kHz_FF(:,3,1,1);
g = E - mean(E);  % feedback path and remove mean value
Lg = length(g);    % length of feedback path
Nfreq = 512;
G = fft(g,Nfreq);

load('mFBPathIRs16kHz_PhoneNear.mat');
Ec = mFBPathIRs16kHz_PhoneNear(:,3,1);
gc = Ec - mean(Ec);  % feedback path and remove mean value
Gc = fft(gc,Nfreq);

TDLy = zeros(Lg,1);            %time-delay vector true feedback path
   
%%%%%%%%%%%%%%%%%%%%%%%%%
%Settings desired signal%
%%%%%%%%%%%%%%%%%%%%%%%%%
%% Desired Signal (incoming signal)

 if in_sig == 0     
    % 0) incoming signal is a white noise
    Var_noise =0.001;
    input = sqrt(Var_noise)*randn(N,1);
 elseif in_sig == 1
    % 1) incoming signal is a synthesized speech
    Var = 1;
    h_den = [1;-2*0.96*cos(3000*2*pi/15750);0.96^2];
%     h_den = [1;-0.8];     % a first-order system
    v = sqrt(Var)*randn(N,1);     % v[k] is white noise with variance one
    input = filter(1,h_den,v);        % speech weighted noise 
 elseif in_sig == 2
    % 2) incoming signal is a real speech segment from NOIZEUS    
    load('HeadMid2_Speech_Vol095_0dgs_m1.mat')
    input1 = HeadMid2_Speech_Vol095_0dgs_m1;
    input = input1(0.5*fs:end);    
    
 else
    % 3) incoming signal is a music 
    load('HeadMid2_Music_Vol095_0dgs_m1.mat')
    input1 = HeadMid2_Music_Vol095_0dgs_m1; % 80s
    input = input1(fs:end);
 end   
 
    input = input./max(abs(input));
    ff = fir1(64,[.025],'high');    
    u_ = filter(ff,1,input);  
    
 u = zeros(N,1); 
for k = 1 : N
    % loop through input signal
    if k <= length(u_)
        u(k) = u_(k);
    else
        u(k) = u_(rem(k,length(u_))+1,1);
    end  
end 

%%%%%%%%%%%%%%%%%%%%%%%%%
%Settings Probe signal  %
%%%%%%%%%%%%%%%%%%%%%%%%%
%% Probe signal w(k)
    Var_P = 0.001;
    if prob_sig == 1
        w = sqrt(Var_P)*randn(N,1);   % w[k] is a white noise;        
    else                            % With probe signal as a white noise        
        w = zeros(N,1);               % Without probe signal
    end
    
 % SNR
    if prob_sig == 1
        estKu = K * u;
        factor = SNRFactor(estKu,w,SNR);
        u = factor * u;
    end
    
%%%%%%%%%%%%%%%%%%%%%%%%%
% Pre-whitening filter  %
%%%%%%%%%%%%%%%%%%%%%%%%%
     
La = 20;
framelength = 0.01*fs;
[AF,AR] = swPemAFCinit(Lg_hat,mu,mu1,mu2,La,framelength,P,delta); 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%initialisation data vectors
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

y = zeros(N,1);         % loudspeaker signal
e_delay = zeros(N+d_k,1);
u_delay = zeros(N+d_k,1);
y_delayfb = zeros(N+d_fb,1);
m = zeros(N,1);         % received microphone signal

% performance parameters
MIS = zeros(N,1);
MSG = zeros(N,1);
ASG = zeros(N,1);

%%%%%%%%%%%%%%%%%%%%%%%%%
% PEM algorithm
%%%%%%%%%%%%%%%%%%%%%%%%%

for k = 2:N 
    % Change the feedback path at sample N/2
    if k == N/2
       g = gc;       
       G = Gc;  
    end
    
       y(k) = K*e_delay(k) + w(k);
       u_delay(k+d_k) = u(k);
     
     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     % Simulated feedback path: computation of microphone signal
     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     
      TDLy = [y(k);TDLy(1:end-1,1)];
      m(k) = u(k) + g'*TDLy;        %received microphone signal
     
      y_delayfb(k+d_fb) = y(k);     

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      %Feedback cancellation
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      [e_delay(k+d_k),AF,AR] = swPemAFC_Modified_SC(m(k),y_delayfb(k),AF,AR,1,1);  

 %  Misaligment of the PEM-AFC
    g_hat = [zeros(d_fb,1);AF.gTD];
    G_hat = fft(g_hat,Nfreq);
    G_tilde = G(1:ceil(Nfreq/2))-G_hat(1:ceil(Nfreq/2));
    
    % Misalignment
    MIS(k) = 20*log10(norm(G_tilde)/norm(G(1:ceil(Nfreq/2))));
    
    % Maximum Stable Gain (MSG)    
    MSG(k) = 20*log10(min(1./abs(G_tilde)));    
    
    % Added Stable Gain (ASG)
    ASG(k) = MSG(k)-20*log10(min(1./abs(G(1:Nfreq/2+1)))); 
end

%% Performace 
    t = linspace(0,(N-1)/fs,N);
     % Misalignment 
     figure(100);
     plot(t,MIS); grid on
     xlabel('Time [s]');
     ylabel('MIS [dB]');
     legend (['swPEMSC, \mu_{1}=' num2str(mu1) ', \mu_{2}=' num2str(mu2)]);
     title(['K=' num2str(Kdb) 'dB, P =' num2str(P) ', dk =' num2str(d_k)]);
    
    % ASG
     figure(300);
     plot(t,ASG);grid on
     xlabel('Time [s]');
     ylabel('ASG [dB]'); 
     legend (['swPEMSCNLMS-PEMSCAPA, \mu_{1}=' num2str(mu1) ', \mu_{2}=' num2str(mu2)]);
     title(['K=' num2str(Kdb) 'dB, P =' num2str(P) ', dk =' num2str(d_k)]);

    figure(400);plot(t,y); grid on; 
    xlabel('Time [s]');
    ylabel('Amplitude');
    title(['K=' num2str(Kdb) 'dB, P =' num2str(P) ', dk =' num2str(d_k)]);

% plot incoming signal
% figure;plot(t,u); grid on; 
% xlabel('Time [s]');
% ylabel('Amplitude');

% Average values of MIS, MSG, ASG

aveMis1 = mean(MIS(2:N/2))
aveASG1 = mean(ASG(2:N/2))
aveMis2 = mean(MIS(N/2+1:end))
aveASG2 = mean(ASG(N/2+1:end))

% Compute PESQ 
PESQ1 = pesq2(u(3:N/2),y(3:N/2),fs)
PESQ2 = pesq2(u(N/2+1:end),y(N/2+1:end),fs)

% save for speech
save('SimResults\swPEMSC_P6_mu1_0.0008_mu2_0.8_K30_La20_spch.mat','MIS','ASG','u','y','e_delay','aveMis1','aveASG1','aveMis2','aveASG2','PESQ1','PESQ2');

% save for music
% save('SimResults\swPEMSC_P6_mu1_0.0008_mu2_0.8_K30_La20_music.mat','MIS','ASG','u','u_delay','y','e_delay','aveMis1','aveASG1','aveMis2','aveASG2');
% soundsc(y,fs);

